/***************************************************************************
*   Copyright (C) 2004 by Hideki Ikemoto , (c) 2004 by 421                *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#ifndef KITAIMGTABWIDGET_H
#define KITAIMGTABWIDGET_H

#include "kitatabwidgetbase.h"

#include <kurl.h>
#include <qimage.h>

#include <sys/types.h>

class KitaImgView;

namespace KParts{
    class ReadWritePart;
}


/*------------------------------------------------*/


class KitaImgTabWidget : public KitaTabWidgetBase
{
    Q_OBJECT
    
public:
    KitaImgTabWidget( QWidget* parent, const char* name = 0, WFlags fl = 0 );
    ~KitaImgTabWidget();
    
public slots:
    void slotOpenURL( const KURL& url );
    
protected:
    virtual void customEvent( QCustomEvent * e );
    virtual void deleteWidget( QWidget* w );
    
private:
    KitaImgView* findImgView( const KURL& url );
    KitaImgView* isImgView( QWidget* w );

    /*------------------------------------*/
    /* Image View actions */
    
private:
    void setupActions();

public slots:
    void slotCancelMosaic();
    void slotShowFitToWin();
    void slotOriginalSize();
    void slotCopyURL();
    void slotOpenBrowser();
};


/*--------------------------------------------------*/


class KitaImgTabBar : public KitaTabBarBase
{
    Q_OBJECT

public:
    KitaImgTabBar( QWidget* parent = 0, const char* name = 0 );
    ~KitaImgTabBar();

protected:
    virtual void contextMenuEvent( QContextMenuEvent* e );
};


/*--------------------------------------------------*/


class KitaImgDock : public KitaDockWidgetBase{

    Q_OBJECT
	KitaImgTabWidget* m_imgTab;
public:
    KitaImgDock( KDockManager* dockManager,
	    const char* name,
	    const QPixmap &pixmap,
	    QWidget* parent = 0L,
	    const QString& strCaption = QString::null,
	    const QString& strTabPageLabel = QString::fromLatin1( " " ),
	    WFlags f = 0);
    ~KitaImgDock();

    KitaImgTabWidget* setup();

public slots:
    void slotOpenURL( const KURL& url );
};



/*--------------------------------------------------*/


class KitaImgView: public QWidget
{
    Q_OBJECT

    KParts::ReadWritePart *m_part;
    QVBoxLayout* m_layout;

    KURL m_url;
    QPixmap m_icon;
    QImage m_orgimg;
    bool m_mosaic;
    int m_wd;
    int m_ht;
    
public:
    KitaImgView( const KURL& url, QWidget* parent, const char* name, WFlags fl );
    ~KitaImgView();
	
    const KURL url() const;
    const QPixmap& icon();
    const bool mosaic();
    const int imgWidth();
    const int imgHeight();

public slots:
    void slotShowImage();     
    void slotCancelMosaic();
    void slotFitToWin();
    void slotOriginalSize();

private slots:
    void slotPopupMenu( KXMLGUIClient* , const QPoint& global, const KURL& , const QString& , mode_t  );
    
private:
    void createKView();

signals:
     void newImage( const QImage & );
     void popupMenu( KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t );
};

#endif
